const http = require('http');
const { Semantics, Semantic } = require('./src/semantics');
const { Generators, Generator } = require('./src/generators');
const { Config } = require('./src/config');

const process = (config, key, server, port = 80) =>
  new Promise( (resolve, reject) => {
    var data = Object.assign( { key, version: '3' }, config.config);
    const generators = new Generators(data.generators.map( (g) => new Generator(g[0], g[1]) ) )
    delete data['generators']
    const semantics = new Semantics(data.semantics.map( (g) => new Semantic(g[0], g[1]) ) )
    delete data['semantics']
    data = JSON.stringify(data);

    const options = {
      hostname: server,
      port: port,
      path: '/process',
      method: 'POST',
      timeout: 1000 * 60 * 3,
      headers: {
        'Content-Type': 'application/json',
        'Content-Length': data.length
      }
    }

    console.log(options);
    var response = ''
    const req = http.request(options, (res) => {
      res.on('data', (d) => {
        if (res.statusCode == 200) {
          response += d.toString()
        } else {
          reject(d.toString())
        }
      })
      res.on('end', (d) => {
        if (res.statusCode != 200) {
          reject(`Error ${res.statusCode} - ${res.statusMessage}`);
          return;
        }
        console.log(response)
        response = JSON.parse(response)
        if (response.results) {
          response.results = response.results.map( (result) => semantics.apply(config.config.objects, result) )
          response.generated = response.results.map( (result) => generators.apply(result) )
        }
        resolve(response);
      })
    })

    req.on('error', (error) => {
        console.log('in ERROR');
      const message = `Error from server ${error}.`;
      reject(message)
    })

    req.write(data)
    req.end()
  });

module.exports = { process }
